@@ -17,15 +17,18 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef LIBKGAPI2_CALENDARSERVICE_H
 #define LIBKGAPI2_CALENDARSERVICE_H
 
 #include "types.h"
+#include "enums.h"
 #include "kgapicalendar_export.h"
 
+#include <QFlags>
+
 namespace KGAPI2
 {
 
@@ -36,11 +39,16 @@ namespace KGAPI2
  */
 namespace CalendarService
 {
+    enum class EventSerializeFlag {
+        Default     =   0,
+        NoID        =   1 << 0
+    };
+    using EventSerializeFlags = QFlags<EventSerializeFlag>;
 
     /**
      * @brief Parses calendar JSON data into Calendar object
      *
-     * @param JsonData
+     * @param jsonData
      */
     KGAPICALENDAR_EXPORT CalendarPtr JSONToCalendar(const QByteArray& jsonData);
 
@@ -72,7 +80,7 @@ namespace CalendarService
      *
      * @param event
      */
-    KGAPICALENDAR_EXPORT QByteArray eventToJSON(const EventPtr& event);
+    KGAPICALENDAR_EXPORT QByteArray eventToJSON(const EventPtr& event, EventSerializeFlags flags = EventSerializeFlag::Default);
 
     /**
      * @brief Parses JSON feed into list of Events
@@ -84,7 +92,7 @@ namespace CalendarService
     KGAPICALENDAR_EXPORT ObjectsList parseEventJSONFeed(const QByteArray& jsonFeed, FeedData& feedData);
 
     /**
-     * @brief Supported API verstion
+     * @brief Supported API version
      */
     KGAPICALENDAR_EXPORT QString APIVersion();
 
@@ -139,15 +147,18 @@ namespace CalendarService
      *
      * @param calendarID ID of calendar in which the event is
      * @param eventID ID of event to update
+     * @param updatesPolicy Whether to send notification to participants
      */
-    KGAPICALENDAR_EXPORT QUrl updateEventUrl(const QString &calendarID, const QString &eventID);
+    KGAPICALENDAR_EXPORT QUrl updateEventUrl(const QString &calendarID, const QString &eventID,
+                                             SendUpdatesPolicy updatesPolicy);
 
     /**
      * @brief Returns URL creating new events.
      *
      * @param calendarID ID of calendar in which to create the event
+     * @param updatesPolicy Whether to send notification to participants
      */
-    KGAPICALENDAR_EXPORT QUrl createEventUrl(const QString &calendarID);
+    KGAPICALENDAR_EXPORT QUrl createEventUrl(const QString &calendarID, SendUpdatesPolicy updatesPolicy);
 
     /**
      * @brief Returns URL for removing events
@@ -167,7 +178,7 @@ namespace CalendarService
     KGAPICALENDAR_EXPORT QUrl moveEventUrl(const QString &sourceCalendar, const QString &destCalendar, const QString &eventID);
 
     /**
-     * @brief Returns URL for freebusy quries.
+     * @brief Returns URL for freebusy queries.
      */
     KGAPICALENDAR_EXPORT QUrl freeBusyQueryUrl();
 
@@ -175,4 +186,6 @@ namespace CalendarService
 
 } // namespace KGAPI
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(KGAPI2::CalendarService::EventSerializeFlags)
+
 #endif // LIBKGAPI2_CALENDARSERVICE_H
