@@ -15,7 +15,7 @@
     GNU Lesser General Public License for more details.
 
     You should have received a copy of the GNU Lesser General Public
-    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
+    License along with this library.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "contactsservice.h"
@@ -52,6 +52,15 @@ namespace Private
     static const QString ContactsBasePath(QStringLiteral("/m8/feeds/contacts"));
     static const QString ContactsGroupBasePath(QStringLiteral("/m8/feeds/groups"));
     static const QString PhotoBasePath(QStringLiteral("/m8/feeds/photos/media"));
+
+    QByteArray addRelSchema(const QByteArray &rel)
+    {
+        if (!rel.startsWith("http://schemas.google.com/g/2005#")) {
+            return "http://schemas.google.com/g/2005#" + rel;
+        } else {
+            return rel;
+        }
+    }
 }
 
 ObjectsList parseJSONFeed(const QByteArray& jsonFeed, FeedData& feedData)
@@ -297,7 +306,7 @@ ObjectPtr Private::JSONToContact(const QVariantMap& data)
     /* Google contact ID */
     contact->setUid(Private::stringFromXMLMap(data, QStringLiteral("id")));
 
-    /* Google ETAG. This can be used to identify if the item was changed remotly */
+    /* Google ETAG. This can be used to identify if the item was changed remotely */
     contact->setEtag(data.value(QStringLiteral("gd$etag")).toString());
 
     /* Date and time when contact was updated on the remote server */
@@ -692,8 +701,8 @@ QByteArray contactToXML(const ContactPtr& contact)
     /* Emails */
     const auto preferredEmail = contact->preferredEmail();
     Q_FOREACH(const auto &email, contact->emailList()) {
-        const auto rels = email.parameters().value(QStringLiteral("TYPE"), { QStringLiteral("http://schemas.google.com/g/2005#home") });
-        auto rel = rels.isEmpty() ? "http://schemas.google.com/g/2005#home" : rels.at(0).toLower().toUtf8();
+        const auto rels = email.parameters().value(QStringLiteral("TYPE"), { QStringLiteral("home") });
+        const auto rel = Private::addRelSchema(rels.isEmpty() ? "home" : rels.at(0).toLower().toUtf8());
         output.append("<gd:email rel='" + rel + "' address='").append(email.mail().toHtmlEscaped().toUtf8()).append("'");
         if (email.mail() == preferredEmail) {
             output.append(" primary=\"true\"");
@@ -850,7 +859,7 @@ ContactPtr XMLToContact(const QByteArray& xmlData)
     QStringList groups;
     ContactPtr contact(new Contact);
 
-    contact->setEtag(entry.at(0).toElement().attribute(QLatin1String("gd:etag")));
+    contact->setEtag(entry.at(0).toElement().attribute(QStringLiteral("gd:etag")));
 
     for (int i = 0; i < data.count(); ++i) {
         const QDomNode n = data.at(i);
